@@ -12,13 +12,13 @@ import ssl
 from datetime import datetime, timedelta
 from json.decoder import JSONDecodeError
 
-import decorator
 import urllib3
+from urllib3.connection import match_hostname as urllib3_match_hostname
 from urllib3.util.ssl_ import ssl_wrap_socket as urllib3_ssl_wrap_socket
 from urllib3.util.ssl_ import wrap_socket as urllib3_wrap_socket
 
 from .compat import basestring, byte_type, decode_from_bytes, encode_to_bytes, text_type
-from .utils import SSL_PROTOCOL, MocketSocketCore, hexdump, hexload, wrap_ssl_socket
+from .utils import SSL_PROTOCOL, MocketSocketCore, get_mocketize, hexdump, hexload
 
 xxh32 = None
 try:
@@ -49,24 +49,41 @@ true_ssl_context = ssl.SSLContext
 true_inet_pton = socket.inet_pton
 true_urllib3_wrap_socket = urllib3_wrap_socket
 true_urllib3_ssl_wrap_socket = urllib3_ssl_wrap_socket
+true_urllib3_match_hostname = urllib3_match_hostname
 
 
-class SuperFakeSSLContext(object):
-    """ For Python 3.6 """
+class SuperFakeSSLContext:
+    """For Python 3.6"""
 
     class FakeSetter(int):
         def __set__(self, *args):
             pass
 
     options = FakeSetter()
-    verify_mode = FakeSetter(ssl.CERT_OPTIONAL)
+    verify_mode = FakeSetter(ssl.CERT_NONE)
 
 
 class FakeSSLContext(SuperFakeSSLContext):
+    DUMMY_METHODS = (
+        "load_default_certs",
+        "load_verify_locations",
+        "set_alpn_protocols",
+    )
     sock = None
     post_handshake_auth = None
+    _check_hostname = False
+
+    @property
+    def check_hostname(self):
+        return self._check_hostname
+
+    @check_hostname.setter
+    def check_hostname(self, *args):
+        self._check_hostname = False
 
     def __init__(self, sock=None, server_hostname=None, _context=None, *args, **kwargs):
+        self._set_dummy_methods()
+
         if isinstance(sock, MocketSocket):
             self.sock = sock
             self.sock._host = server_hostname
@@ -78,13 +95,12 @@ class FakeSSLContext(SuperFakeSSLContext):
         elif isinstance(sock, int) and true_ssl_context:
             self.context = true_ssl_context(sock)
 
-    @staticmethod
-    def load_default_certs(*args, **kwargs):
-        pass
+    def _set_dummy_methods(self):
+        def dummy_method(*args, **kwargs):
+            pass
 
-    @staticmethod
-    def load_verify_locations(*args, **kwargs):
-        pass
+        for m in self.DUMMY_METHODS:
+            setattr(self, m, dummy_method)
 
     @staticmethod
     def wrap_socket(sock=sock, *args, **kwargs):
@@ -92,7 +108,8 @@ class FakeSSLContext(SuperFakeSSLContext):
         sock._secure_socket = True
         return sock
 
-    def wrap_bio(self, incoming, outcoming, *args, **kwargs):
+    @staticmethod
+    def wrap_bio(incoming, outcoming, *args, **kwargs):
         ssl_obj = MocketSocket()
         ssl_obj._host = kwargs["server_hostname"]
         return ssl_obj
@@ -114,7 +131,7 @@ def _hash_request(h, req):
     return h(encode_to_bytes("".join(sorted(req.split("\r\n"))))).hexdigest()
 
 
-class MocketSocket(object):
+class MocketSocket:
     timeout = None
     _fd = None
     family = None
@@ -141,16 +158,6 @@ class MocketSocket(object):
         self._truesocket_recording_dir = None
         self.kwargs = kwargs
 
-        sock = kwargs.get("sock")
-        if sock is not None:
-            self.__dict__ = dict(sock.__dict__)
-
-            self.true_socket = wrap_ssl_socket(
-                true_ssl_socket,
-                self.true_socket,
-                true_ssl_context(protocol=SSL_PROTOCOL),
-            )
-
     def __unicode__(self):  # pragma: no cover
         return str(self)
 
@@ -185,7 +192,8 @@ class MocketSocket(object):
     def settimeout(self, timeout):
         self.timeout = timeout
 
-    def getsockopt(self, level, optname, buflen=None):
+    @staticmethod
+    def getsockopt(level, optname, buflen=None):
         return socket.SOCK_STREAM
 
     def do_handshake(self):
@@ -226,7 +234,8 @@ class MocketSocket(object):
     def write(self, data):
         return self.send(encode_to_bytes(data))
 
-    def fileno(self):
+    @staticmethod
+    def fileno():
         Mocket.r_fd, Mocket.w_fd = os.pipe()
         return Mocket.r_fd
 
@@ -247,15 +256,19 @@ class MocketSocket(object):
             entry = self.get_entry(data)
 
         if entry:
-            entry.collect(data)
-            response = entry.get_response()
+            consume_response = entry.collect(data)
+            if consume_response is not False:
+                response = entry.get_response()
+            else:
+                response = None
         else:
             response = self.true_sendall(data, *args, **kwargs)
 
-        self.fd.seek(0)
-        self.fd.write(response)
-        self.fd.truncate()
-        self.fd.seek(0)
+        if response is not None:
+            self.fd.seek(0)
+            self.fd.write(response)
+            self.fd.truncate()
+            self.fd.seek(0)
 
     def read(self, buffersize):
         return self.fd.read(buffersize)
@@ -323,16 +336,10 @@ class MocketSocket(object):
             host = true_gethostbyname(host)
 
             if isinstance(self.true_socket, true_socket) and self._secure_socket:
-                try:
-                    self = MocketSocket(sock=self)
-                except TypeError:
-                    ssl_context = self.kwargs.get("ssl_context")
-                    server_hostname = self.kwargs.get("server_hostname")
-                    self.true_socket = true_ssl_context.wrap_socket(
-                        self=ssl_context,
-                        sock=self.true_socket,
-                        server_hostname=server_hostname,
-                    )
+                self.true_socket = true_urllib3_ssl_wrap_socket(
+                    self.true_socket,
+                    **self.kwargs,
+                )
 
             try:
                 self.true_socket.connect((host, port))
@@ -388,7 +395,7 @@ class MocketSocket(object):
         self._fd = None
 
     def __getattr__(self, name):
-        """ Do nothing catchall function, for methods like close() and shutdown() """
+        """Do nothing catchall function, for methods like close() and shutdown()"""
 
         def do_nothing(*args, **kwargs):
             pass
@@ -396,7 +403,7 @@ class MocketSocket(object):
         return do_nothing
 
 
-class Mocket(object):
+class Mocket:
     _entries = collections.defaultdict(list)
     _requests = []
     _namespace = text_type(id(_entries))
@@ -479,6 +486,9 @@ class Mocket(object):
         urllib3.connection.ssl_wrap_socket = urllib3.connection.__dict__[
             "ssl_wrap_socket"
         ] = FakeSSLContext.wrap_socket
+        urllib3.connection.match_hostname = urllib3.connection.__dict__[
+            "match_hostname"
+        ] = lambda cert, hostname: None
         if pyopenssl_override:  # pragma: no cover
             # Take out the pyopenssl version - use the default implementation
             extract_from_urllib3()
@@ -506,6 +516,9 @@ class Mocket(object):
         urllib3.connection.ssl_wrap_socket = urllib3.connection.__dict__[
             "ssl_wrap_socket"
         ] = true_urllib3_ssl_wrap_socket
+        urllib3.connection.match_hostname = urllib3.connection.__dict__[
+            "match_hostname"
+        ] = true_urllib3_match_hostname
         Mocket.reset()
         if pyopenssl_override:  # pragma: no cover
             # Put the pyopenssl version back in place
@@ -521,18 +534,19 @@ class Mocket(object):
 
     @classmethod
     def assert_fail_if_entries_not_served(cls):
-        """ Mocket checks that all entries have been served at least once. """
+        """Mocket checks that all entries have been served at least once."""
         assert all(
             entry._served for entry in itertools.chain(*cls._entries.values())
         ), "Some Mocket entries have not been served"
 
 
-class MocketEntry(object):
+class MocketEntry:
     class Response(byte_type):
         @property
         def data(self):
             return self
 
+    response_index = 0
     request_cls = str
     response_cls = Response
     responses = None
@@ -541,27 +555,27 @@ class MocketEntry(object):
     def __init__(self, location, responses):
         self._served = False
         self.location = location
-        self.response_index = 0
 
         if not isinstance(responses, collections_abc.Iterable) or isinstance(
             responses, basestring
         ):
             responses = [responses]
 
-        self.responses = []
-        for r in responses:
-            if isinstance(r, BaseException):
-                pass
-            elif not getattr(r, "data", False):
-                if isinstance(r, text_type):
-                    r = encode_to_bytes(r)
-                r = self.response_cls(r)
-            self.responses.append(r)
+        if not responses:
+            self.responses = [self.response_cls(encode_to_bytes(""))]
         else:
-            if not responses:
-                self.responses = [self.response_cls(encode_to_bytes(""))]
+            self.responses = []
+            for r in responses:
+                if isinstance(r, BaseException):
+                    pass
+                elif not getattr(r, "data", False):
+                    if isinstance(r, text_type):
+                        r = encode_to_bytes(r)
+                    r = self.response_cls(r)
+                self.responses.append(r)
 
-    def can_handle(self, data):
+    @staticmethod
+    def can_handle(data):
         return True
 
     def collect(self, data):
@@ -581,53 +595,62 @@ class MocketEntry(object):
         return response.data
 
 
-class Mocketizer(object):
+class Mocketizer:
     def __init__(self, instance=None, namespace=None, truesocket_recording_dir=None):
         self.instance = instance
         self.truesocket_recording_dir = truesocket_recording_dir
         self.namespace = namespace or text_type(id(self))
 
-    def __enter__(self):
+    def enter(self):
         Mocket.enable(
             namespace=self.namespace,
             truesocket_recording_dir=self.truesocket_recording_dir,
         )
         if self.instance:
             self.check_and_call("mocketize_setup")
+
+    def __enter__(self):
+        self.enter()
         return self
 
-    def __exit__(self, type, value, tb):
+    def exit(self):
         if self.instance:
             self.check_and_call("mocketize_teardown")
         Mocket.disable()
 
+    def __exit__(self, type, value, tb):
+        self.exit()
+
+    async def __aenter__(self, *args, **kwargs):
+        self.enter()
+        return self
+
+    async def __aexit__(self, *args, **kwargs):
+        self.exit()
+
     def check_and_call(self, method):
         method = getattr(self.instance, method, None)
         if callable(method):
             method()
 
-    @classmethod
-    def wrap(cls, test=None, truesocket_recording_dir=None):
-        def wrapper(t, *args, **kw):
-            instance = args[0] if args else None
-            namespace = None
-            if truesocket_recording_dir:
-                namespace = ".".join(
-                    (
-                        instance.__class__.__module__,
-                        instance.__class__.__name__,
-                        t.__name__,
-                    )
-                )
-            with cls(
-                instance,
-                namespace=namespace,
-                truesocket_recording_dir=truesocket_recording_dir,
-            ):
-                t(*args, **kw)
-            return wrapper
 
-        return decorator.decorator(wrapper, test)
+def wrapper(test, cls=Mocketizer, truesocket_recording_dir=None, *args, **kwargs):
+    instance = args[0] if args else None
+    namespace = None
+    if truesocket_recording_dir:
+        namespace = ".".join(
+            (
+                instance.__class__.__module__,
+                instance.__class__.__name__,
+                test.__name__,
+            )
+        )
+    with cls(
+        instance,
+        namespace=namespace,
+        truesocket_recording_dir=truesocket_recording_dir,
+    ):
+        return test(*args, **kwargs)
 
 
-mocketize = Mocketizer.wrap
+mocketize = get_mocketize(wrapper_=wrapper)
